/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;

/**
 * ArchiveScanner.
 * @author Gael Lalire
 */
public abstract class ArchiveScanner {

    /**
     * the classes in archive to scan.
     */
    private List<URL> scannableURL;

    /**
     * a list of configurator.
     */
    private List<IArchiveConfigurator> archiveConfigurators;

    /**
     * true if endScan has been called.
     */
    private boolean scanEnded;

    /**
     * the archive helper.
     */
    private IArchiveHelper archiveHelper;

    /**
     * Constructor.
     * @param archive the archive to scan
     */
    public ArchiveScanner(final IArchive archive) {
        this(new DefaultArchiveHelper(archive));
    }

    /**
     * Constructor with an Archive Helper.
     * @param archiveHelper an archive helper
     */
    public ArchiveScanner(final IArchiveHelper archiveHelper) {
        this.archiveHelper = archiveHelper;
        archiveConfigurators = new ArrayList<IArchiveConfigurator>();
        scanEnded = false;
    }

    /**
     * prepare the scan.
     * @throws ScanException when resources access failed
     */
    private void prepareScan() throws ScanException {
        try {
            scannableURL = archiveHelper.listClassURL();
        } catch (ArchiveException e) {
            throw new ScanException("Error while analyzing archive '" + archiveHelper.getArchive().getName() + "'", e);
        }
    }

    /**
     * add a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public final void addArchiveConfigurator(final IArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.add(archiveConfigurator);
    }

    /**
     * remove a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public final void removeArchiveConfigurator(final IArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.remove(archiveConfigurator);
    }

    /**
     * scan the class contained in url.
     * @param url the url
     * @throws ScanException if error occurs
     */
    protected abstract void doScan(final URL url) throws ScanException;

    /**
     * scan all resources in archive and close the archive.
     * @throws ScanException if error occurs
     */
    public final void scanAll() throws ScanException {
        scanAll(true);
    }

    /**
     * scan all resources in archive.
     * @param closeArchive close the archive
     * @throws ScanException if error occurs
     */
    public final void scanAll(final boolean closeArchive) throws ScanException {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        if (scannableURL == null) {
            prepareScan();
        }
        for (URL url : scannableURL) {
            doScan(url);
        }
        endScan(closeArchive);
    }

    /**
     * scan all resources in archive contained in collection.
     * @param classNameCollection collection of class name
     * @throws ScanException if error occurs
     */
    public final void scanCollection(final Collection<String> classNameCollection) throws ScanException {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        if (scannableURL == null) {
            prepareScan();
        }
        for (String className : classNameCollection) {
            try {
                URL url = archiveHelper.getClassURL(className);
                if (url != null && scannableURL.contains(url)) {
                    doScan(url);
                    scannableURL.remove(url);
                }
            } catch (ArchiveException e) {
                throw new ScanException("Error while analyzing archive '" + archiveHelper.getArchive().getName() + "'", e);
            }
        }
    }

    /**
     * End the scan and close the archive.
     */
    public final void endScan() {
        endScan(true);
    }

    /**
     * End the scan.
     * @param closeArchive if true then the archive will be closed
     */
    public final void endScan(final boolean closeArchive) {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        if (scannableURL != null) {
            scannableURL.clear();
        }
        if (closeArchive) {
            archiveHelper.getArchive().close();
        }
        scanEnded = true;
    }

    /**
     * @return the archive
     */
    protected final IArchive getArchive() {
        return archiveHelper.getArchive();
    }

    /**
     * @return the configurator list
     */
    protected final List<IArchiveConfigurator> getArchiveConfigurators() {
        return archiveConfigurators;
    }

}
