/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassMetadataConfigurator.java 3279 2008-05-21 16:00:00Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.configurator.metadata;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.scan.api.configurator.BasicConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificArchiveConfigurator;
import org.ow2.util.scan.api.configurator.metadata.specific.ISpecificClassConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * ClassMetadataConfigurator is a configurator of class specific to ClassMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class ClassMetadataConfigurator<
    C extends IClassMetadata<C, M, F>,
    M extends IMethodMetadata<C, M, F>,
    F extends IFieldMetadata<C, M, F>>
        extends BasicConfigurator implements IClassConfigurator {

    /**
     * The class metadata.
     */
    private C classMetadata;

    /**
     * List of specific class configurator.
     */
    private List<ISpecificClassConfigurator<C, M, F>> specificClassConfiguratorList;

    /**
     * @return the class metadata.
     */
    protected C getClassMetadata() {
        return classMetadata;
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     */
    public ClassMetadataConfigurator(final C classMetadata) {
        this.classMetadata = classMetadata;
        specificClassConfiguratorList = new ArrayList<ISpecificClassConfigurator<C, M, F>>();
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     * @param specificArchiveConfiguratorList a list of specific archive configurator
     */
    public ClassMetadataConfigurator(final C classMetadata,
            final List<? extends ISpecificArchiveConfigurator<C, M, F>> specificArchiveConfiguratorList) {
        this.classMetadata = classMetadata;
        specificClassConfiguratorList = new ArrayList<ISpecificClassConfigurator<C, M, F>>();
        for (ISpecificArchiveConfigurator<C, M, F> specificArchiveConfigurator : specificArchiveConfiguratorList) {
            specificClassConfiguratorList.add(specificArchiveConfigurator.createSpecificClassConfigurator(classMetadata));
        }
    }

    /**
     * Provide a method used by ClassConfigurator.
     * @param jMethod signature of method
     * @return the configurator
     */
    public final IMethodConfigurator createMethodConfigurator(final JMethod jMethod) {
        return createMethodMetadataConfigurator(jMethod);
    }

    /**
     * You have to override this method.
     * @param jMethod signature of method
     * @return the configurator
     */
    protected abstract IMethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod);

    /**
     * Provide a method used by ClassConfigurator.
     * @param jField field signature
     * @return the configurator
     */
    public final IFieldConfigurator createFieldConfigurator(final JField jField) {
        return createFieldMetadataConfigurator(jField);
    }

    /**
     * You have to override this method.
     * @param jField signature of field
     * @return the configurator
     */
    protected abstract IFieldConfigurator createFieldMetadataConfigurator(final JField jField);

    /**
     * call specific configuration complete.
     */
    @Override
    public void configurationComplete() {
        for (ISpecificClassConfigurator<C, M, F> specificClassConfigurator : specificClassConfiguratorList) {
            specificClassConfigurator.configurationComplete();
        }
        super.configurationComplete();
    }

    /**
     * @return a list
     */
    protected List<ISpecificClassConfigurator<C, M, F>> getSpecificClassConfiguratorList() {
        return specificClassConfiguratorList;
    }

}
