/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: SpecificClassMetadata.java 3279 2008-05-21 16:00:00Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.metadata.specific;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * @param <SC> type of specific class metadata
 * @param <SM> type of specific method metadata
 * @param <SF> type of specific field metadata
 * @author Gael Lalire
 */
public abstract class SpecificClassMetadata<
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>>
        implements ISpecificClassMetadata<SC, SM, SF> {

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, SM> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, SF> fieldsAnnotationMetadata = null;

    /**
     * Constructor.
     */
    public SpecificClassMetadata() {
        this.methodsAnnotationMetadata = new HashMap<JMethod, SM>();
        this.fieldsAnnotationMetadata = new HashMap<JField, SF>();
    }

    /**
     * {@inheritDoc}
     */
    public void addSpecificFieldMetadata(final SF fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            throw new IllegalStateException();
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public void addSpecificMethodMetadata(final SM methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public SF getSpecificFieldMetadata(final JField field) {
        return fieldsAnnotationMetadata.get(field);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends SF> getSpecificFieldMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }

    /**
     * {@inheritDoc}
     */
    public SM getSpecificMethodMetadata(final JMethod method) {
        return methodsAnnotationMetadata.get(method);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends SM> getSpecificMethodMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends SF> searchSpecificFieldMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<SF> list = new ArrayList<SF>();
        for (SF field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getJField().getName())) {
                list.add(field);
            }
        }
        return list;
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends SM> searchSpecificMethodMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<SM> list = new ArrayList<SM>();
        for (SM method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getJMethod().getName())) {
                list.add(method);
            }
        }
        return list;
    }


}
